************************************************************************
*     ---------------------------------
      REAL FUNCTION COHCRSNEBERGER(E,IPAR,IC)
*     ---------------------------------
*
*     (Purpose)
*       CROSS SECTION / C12 NUCLEUS ( 10**(-38) CM**2 )
*       It's coherent with the plots Fig3 in Berger and Sehgal paper.
*       
*
*     (Input)
*       E     : INCIDENT NEUTRINO ENERGY ( GEV )
*       IPAR  : INCIDENT NEUTRINO ID
*       IC    : INTERACTION MODE
*                1 : C.C.
*                0 : N.C.
*     (Output)
*       COHCRSNE :  CROSS SECTION / C12 NUCLEUS ( 10**(-38) CM**2 )
*      
*     (Creation Date and Author)
*       1994.08.30 ; K.Okumura
*       1996.02.05 ; K.Kaneyuki   COHCRS -> COHCRSNE
*       1996.12.20 ; K.Okumura    add cross section from 100GeV to 1000GeV
*       2007.01.08 ; G.Mitsuka    Taking into account lepton mass in CC,
*                                 and rename into cohcrsnerein.F
*       2007.11.05 ; G.Mitsuka    support target nucleus besides 16O
*
*       2009.11.18 ; Y.Hayato     Recalculated with lepton mass corr.
*
*     (Comment)
*
************************************************************************

      IMPLICIT NONE
#include "necard.h"
      INTEGER IPAR,IC,I
      REAL E,WIDTH,EEND(2),SIGEND(2),SLOPE,ESTART
C     Charged Current
      REAL SIG1CCE(0:100),SIG2CCE(0:90),SIG3CCE(0:90),SIG4CCE(0:90)
      REAL SIG1CCM(0:100),SIG2CCM(0:90),SIG3CCM(0:90),SIG4CCM(0:90)
      REAL SIG1CCT(0:100),SIG2CCT(0:90),SIG3CCT(0:90),SIG4CCT(0:90)
      REAL S0CCE(50),S1CCE(50),S2CCE(50),S3CCE(40)
      REAL S4CCE(50),S5CCE(40),S6CCE(50),S7CCE(40)
      REAL S0CCM(50),S1CCM(50),S2CCM(50),S3CCM(40)
      REAL S4CCM(50),S5CCM(40),S6CCM(50),S7CCM(40)
      REAL S0CCT(50),S1CCT(50),S2CCT(50),S3CCT(40)
      REAL S4CCT(50),S5CCT(40),S6CCT(50),S7CCT(40)
C     Electron neutrino
      EQUIVALENCE (SIG2CCE(0),SIG1CCE(100))
      EQUIVALENCE (SIG3CCE(0),SIG2CCE(90))
      EQUIVALENCE (SIG4CCE(0),SIG3CCE(90))
      EQUIVALENCE (SIG1CCE(1), S0CCE(1))
      EQUIVALENCE (SIG1CCE(51),S1CCE(1))
      EQUIVALENCE (SIG2CCE(1), S2CCE(1))
      EQUIVALENCE (SIG2CCE(51),S3CCE(1))
      EQUIVALENCE (SIG3CCE(1), S4CCE(1))
      EQUIVALENCE (SIG3CCE(51),S5CCE(1))
      EQUIVALENCE (SIG4CCE(1), S6CCE(1))
      EQUIVALENCE (SIG4CCE(51),S7CCE(1))
C     Muon neutrino
      EQUIVALENCE (SIG2CCM(0),SIG1CCM(100))
      EQUIVALENCE (SIG3CCM(0),SIG2CCM(90))
      EQUIVALENCE (SIG4CCM(0),SIG3CCM(90))
      EQUIVALENCE (SIG1CCM(1), S0CCM(1))
      EQUIVALENCE (SIG1CCM(51),S1CCM(1))
      EQUIVALENCE (SIG2CCM(1), S2CCM(1))
      EQUIVALENCE (SIG2CCM(51),S3CCM(1))
      EQUIVALENCE (SIG3CCM(1), S4CCM(1))
      EQUIVALENCE (SIG3CCM(51),S5CCM(1))
      EQUIVALENCE (SIG4CCM(1), S6CCM(1))
      EQUIVALENCE (SIG4CCM(51),S7CCM(1))
C     Tau neutrino
      EQUIVALENCE (SIG2CCT(0),SIG1CCT(100))
      EQUIVALENCE (SIG3CCT(0),SIG2CCT(90))
      EQUIVALENCE (SIG4CCT(0),SIG3CCT(90))
      EQUIVALENCE (SIG1CCT(1), S0CCT(1))
      EQUIVALENCE (SIG1CCT(51),S1CCT(1))
      EQUIVALENCE (SIG2CCT(1), S2CCT(1))
      EQUIVALENCE (SIG2CCT(51),S3CCT(1))
      EQUIVALENCE (SIG3CCT(1), S4CCT(1))
      EQUIVALENCE (SIG3CCT(51),S5CCT(1))
      EQUIVALENCE (SIG4CCT(1), S6CCT(1))
      EQUIVALENCE (SIG4CCT(51),S7CCT(1))

      ! Neutral Current
      REAL SIG1NC(0:100),SIG2NC(0:90),SIG3NC(0:90),SIG4NC(0:90)
      REAL S0NC(50),S1NC(50),S2NC(50),S3NC(40)
      REAL S4NC(50),S5NC(40),S6NC(50),S7NC(40)
      EQUIVALENCE (SIG2NC(0),SIG1NC(100))
      EQUIVALENCE (SIG3NC(0),SIG2NC(90))
      EQUIVALENCE (SIG4NC(0),SIG3NC(90))
      EQUIVALENCE (SIG1NC(1), S0NC(1))
      EQUIVALENCE (SIG1NC(51),S1NC(1))
      EQUIVALENCE (SIG2NC(1), S2NC(1))
      EQUIVALENCE (SIG2NC(51),S3NC(1))
      EQUIVALENCE (SIG3NC(1), S4NC(1))
      EQUIVALENCE (SIG3NC(51),S5NC(1))
      EQUIVALENCE (SIG4NC(1), S6NC(1))
      EQUIVALENCE (SIG4NC(51),S7NC(1))

C     Table below assumes 16O nuclei, for other nuclei     
C     ScalingFactor:(msss#/16)^(1/3) should be applied
      real ScalingFactor

C     Charged Current
C     ===============
C     Electron neutrino
C
C     0 - 1GeV - DONE
C
       DATA S0CCE/0 , 0       ,0          ,0 ,
     & 0          ,0          ,0          ,0 ,
     & 0          ,0          ,0          ,0 , 
     & 0          ,0          ,2.06624e-11,7.05462e-08 ,
     & 9.24709e-07,4.33333e-06,1.27637e-05,2.93844e-05 ,
     & 5.76543e-05,0.000100893,0.00016229 ,0.000247314 ,
     & 0.000364466,0.000516015,0.0007233  ,0.000984659 ,
     & 0.00130554 ,0.00168965 ,0.0021271  ,0.00262959 ,
     & 0.00318598 ,0.00380246 ,0.00447607 ,0.0052003 ,
     & 0.00597076 ,0.0067718  ,0.00760165 ,0.00855474 ,
     & 0.00946281 ,0.0104141  ,0.0113871  ,0.0123547 ,
     & 0.0133624  ,0.0143702  ,0.0154215  ,0.016487 ,
     & 0.0175789  ,0.0186804/

       DATA S1CCE/ 0.019796 , 0.0208706 , 0.0220821 , 0.0232224 ,
     & 0.0243549 , 0.0255032 , 0.0266607 , 0.0278232 ,
     & 0.028951  , 0.0301358 , 0.0313368 , 0.03248 ,
     & 0.033689  , 0.034788  , 0.0359592 , 0.0369655 ,
     & 0.038106  , 0.039296  , 0.040466  , 0.041654 ,
     & 0.0427653 , 0.0438806 , 0.0450199 , 0.0461046 ,
     & 0.0472379 , 0.048335  , 0.0493597 , 0.0506249 ,
     & 0.0517156 , 0.052866  , 0.0539596 , 0.0550011 ,
     & 0.0560206 , 0.0569911 , 0.0580869 , 0.0590473 ,
     & 0.06003   , 0.0609768 , 0.0619811 , 0.0626027 ,
     & 0.0635707 , 0.0645163 , 0.0655088 , 0.0664805 ,
     & 0.0673852 , 0.0683463 , 0.0692566 , 0.0701563 ,
     & 0.0710719 , 0.0720128/
C      
C     1 - 10GeV - DONE
C
       DATA S2CCE/0.0733394 , 0.0825488 , 0.0913459 , 0.0999169 ,
     & 0.108177 , 0.116201 , 0.123996 , 0.13185 ,
     & 0.139762 , 0.147067 , 0.154241 , 0.161498 ,
     & 0.168605 , 0.175679 , 0.182766 , 0.189739 ,
     & 0.196601 , 0.203814 , 0.210423 , 0.217128 ,
     & 0.223645 , 0.22999  , 0.236198 , 0.242492 ,
     & 0.248796 , 0.254879 , 0.25988  , 0.266075 ,
     & 0.272076 , 0.277895 , 0.283759 , 0.28956 ,
     & 0.295292 , 0.301351 , 0.307099 , 0.313247 ,
     & 0.318519 , 0.324395 , 0.329765 , 0.335156 ,
     & 0.340613 , 0.345941 , 0.351581 , 0.358667 ,
     & 0.363923 , 0.369097 , 0.374277 , 0.379249 ,
     & 0.384541 , 0.389793/

       DATA S3CCE/ 0.394738 , 0.399398 , 0.40267 , 0.407443 ,
     & 0.412233 , 0.417152 , 0.421973 , 0.426749 ,
     & 0.431455 , 0.4362   , 0.440857 , 0.446439 , 
     & 0.451168 , 0.455926 , 0.460495 , 0.465146 ,
     & 0.469643 , 0.474362 , 0.478898 , 0.479978 ,
     & 0.484455 , 0.488914 , 0.493364 , 0.49745 ,
     & 0.501664 , 0.505958 , 0.510259 , 0.514488 ,
     & 0.520503 , 0.524741 , 0.528789 , 0.532967 ,
     & 0.536969 , 0.540736 , 0.544812 , 0.548702 ,
     & 0.552536 , 0.555903 , 0.559774 , 0.563708/ 
C
C     10 - 29 GeV with integration | 30 - 100GeV interpolation
C
       DATA S4CCE/0.569482 , 0.607702 , 0.643806 , 0.678178 ,
     & 0.711602 , 0.743057 , 0.77333  , 0.802924 , 
     & 0.830777 , 0.85779  , 0.884255 , 0.908737 ,
     & 0.934036 , 0.95752  , 0.980862 , 1.003 ,
     & 1.02543  , 1.04586  , 1.0669   , 1.08647 ,
     & 1.10738  , 1.12695  , 1.14462  , 1.16301 ,
     & 1.18187  , 1.1986   , 1.21611  , 1.23222 ,
     & 1.24914  , 1.26588  , 1.28096  , 1.29675 ,
     & 1.31223  , 1.32628  , 1.34104  , 1.35519 ,
     & 1.3699   , 1.38384  , 1.39689  , 1.41049 ,
     & 1.42368  , 1.43773  , 1.45003  , 1.46272 ,
     & 1.47567  , 1.48809  , 1.50046  , 1.51177 ,
     & 1.52405  , 1.53485/

       DATA S5CCE/ 1.54624 , 1.55841 , 1.56766 , 1.57995 ,
     & 1.59043 , 1.6021  , 1.61182 , 1.62269 ,
     & 1.63267 , 1.64289 , 1.65277 , 1.66268 ,
     & 1.67262 , 1.68323 , 1.69204 , 1.70144 ,
     & 1.7107  , 1.72053 , 1.72875 , 1.73751 ,
     & 1.74744 , 1.75645 , 1.76556 , 1.77466 ,
     & 1.78291 , 1.79004 , 1.79949 , 1.80769 ,
     & 1.8163  , 1.82464 , 1.83232 , 1.84079 ,
     & 1.84854 , 1.85672 , 1.86419 , 1.871 ,
     & 1.87966 , 1.88651 , 1.89561 , 1.90055/ 
C
C     100 - 1000GeV interpolation
C      
       DATA S6CCE/1.90941 , 1.97942 , 2.04612 , 2.10747 ,
     & 2.16426 , 2.21595 , 2.26569 , 2.31344 ,
     & 2.35816 , 2.40113 , 2.4401  , 2.48302 ,
     & 2.51876 , 2.55491 , 2.58788 , 2.62174 ,
     & 2.65164 , 2.68122 , 2.71081 , 2.74204 ,
     & 2.77112 , 2.79542 , 2.82371 , 2.84843 ,
     & 2.87448 , 2.90066 , 2.91567 , 2.94338 ,
     & 2.96545 , 2.98664 , 3.00889 , 3.02727 ,
     & 3.04849 , 3.0704  , 3.08873 , 3.1053 ,
     & 3.12434 , 3.14271 , 3.15711 , 3.1777 ,
     & 3.19226 , 3.21055 , 3.22365 , 3.24523 ,
     & 3.26113 , 3.27317 , 3.28791 , 3.30548 ,
     & 3.32152 , 3.33107/

       DATA S7CCE/ 3.34344 , 3.36163 , 3.37686 , 3.39064 ,
     & 3.40651 , 3.4147  , 3.43295 , 3.44131 ,
     & 3.45245 , 3.46598 , 3.47628 , 3.49524 ,
     & 3.50555 , 3.51407 , 3.52817 , 3.5384 ,
     & 3.55    , 3.55521 , 3.57283 , 3.58512 ,
     & 3.58896 , 3.60193 , 3.61182 , 3.62597 ,
     & 3.63325 , 3.64212 , 3.65587 , 3.66577 ,
     & 3.677   , 3.68595 , 3.6992  , 3.70613 ,
     & 3.71387 , 3.72356 , 3.7393  , 3.73538 ,
     & 3.74636 , 3.75993 , 3.77186 , 3.77449/ 
C
C     Muon neutrino
C
C     0 - 1GeV - DONE
C
       DATA S0CCM/ 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 ,           2.80962e-08 , 6.73652e-07 , 3.43775e-06 ,
     & 9.8514e-06  , 2.36836e-05 , 4.68129e-05 , 8.21509e-05 ,
     & 0.000134751 , 0.000213339 , 0.000335681 , 0.00051385 ,
     & 0.000757916 , 0.001049    , 0.00139919  , 0.00177505 ,
     & 0.00222381  , 0.00272045  , 0.00326836  , 0.00386312 ,
     & 0.00449158  , 0.00516276  , 0.00588499  , 0.00662113 ,
     & 0.007363    , 0.00815189/

       DATA S1CCM/ 0.00897592 , 0.00984599 , 0.0108308 , 0.0117361 ,
     & 0.0126884 , 0.013647  , 0.0146119 , 0.0155622 ,
     & 0.0165571 , 0.0175513 , 0.0185635 , 0.0195585 ,
     & 0.020601  , 0.0216693 , 0.0228502 , 0.0238949 ,
     & 0.024928  , 0.0260689 , 0.0271489 , 0.0281961 ,
     & 0.0292133 , 0.0303097 , 0.0314401 , 0.0324747 ,
     & 0.0335716 , 0.0345855 , 0.0356677 , 0.0365473 ,
     & 0.0375172 , 0.03861   , 0.0396495 , 0.0406836 ,
     & 0.0417522 , 0.0427061 , 0.043709  , 0.0447376 ,
     & 0.0457626 , 0.0467697 , 0.0477686 , 0.0486535 ,
     & 0.0495517 , 0.0504642 , 0.0513905 , 0.0524509 ,
     & 0.0534775 , 0.0543148 , 0.0552882 , 0.0562748 ,
     & 0.057269  , 0.0582281/       
C      
C     1 - 10GeV - DONE
C       
       DATA S2CCM/ 0.0591705 , 0.06799 , 0.0770802 , 0.0854772 ,
     & 0.0937389 , 0.101788 , 0.109741 , 0.117172 ,
     & 0.124729  , 0.132472 , 0.139876 , 0.146981 ,
     & 0.154296  , 0.161225 , 0.168491 , 0.175098 ,
     & 0.181762  , 0.188639 , 0.195439 , 0.20196 ,
     & 0.208505  , 0.214618 , 0.22127  , 0.227538 ,
     & 0.233433  , 0.239623 , 0.244125 , 0.249876 ,
     & 0.25578   , 0.261585 , 0.267856 , 0.273682 ,
     & 0.279537  , 0.285207 , 0.291716 , 0.297339 ,
     & 0.302934  , 0.308607 , 0.313991 , 0.319519 ,
     & 0.324926  , 0.329995 , 0.335356 , 0.34341 ,
     & 0.348728  , 0.354015 , 0.359246 , 0.364199 , 
     & 0.369081  , 0.37412/

       DATA S3CCM/ 0.379227 , 0.38435 , 0.386593 , 0.391618 ,
     & 0.396193 , 0.401036 , 0.405849 , 0.410664 ,
     & 0.415322 , 0.420141 , 0.424866 , 0.429239 ,
     & 0.433783 , 0.438418 , 0.442796 , 0.447342 ,
     & 0.451956 , 0.456276 , 0.460553 , 0.463412 ,
     & 0.467671 , 0.471775 , 0.476145 , 0.480425 ,
     & 0.484612 , 0.488942 , 0.493239 , 0.497453 ,
     & 0.503485 , 0.507576 , 0.511732 , 0.515889 ,
     & 0.519932 , 0.523733 , 0.527965 , 0.531711 ,
     & 0.535216 , 0.539409 , 0.542986 , 0.546682/
C      
C     10 - 100GeV
C      
       DATA S4CCM/ 0.552598 , 0.590206 , 0.62646 , 0.660854 ,
     & 0.693699 , 0.725269 , 0.755277 , 0.784667 ,
     & 0.812719 , 0.839903 , 0.866279 , 0.8909 ,
     & 0.915703 , 0.940003 , 0.962656 , 0.984936 ,
     & 1.00614  , 1.02876  , 1.0482   , 1.06893 ,
     & 1.08834  , 1.108    , 1.12661  , 1.14436 ,
     & 1.16224  , 1.18053  , 1.19705  , 1.21362 ,
     & 1.23004  , 1.24683  , 1.26147  , 1.27755 ,
     & 1.29265  , 1.30747  , 1.32175  , 1.3371 ,
     & 1.3506   , 1.3649   , 1.37877  , 1.39224 ,
     & 1.40536  , 1.41767  , 1.43009  , 1.44338 ,
     & 1.45675  , 1.46872  , 1.48135  , 1.49161 ,
     & 1.50419  , 1.51524/

       DATA S5CCM/ 1.5274 , 1.53951 , 1.54939 , 1.56107 ,
     & 1.57084 , 1.58265 , 1.5924  , 1.60307 , 
     & 1.61362 , 1.62336 , 1.6331  , 1.64259 ,
     & 1.65289 , 1.66207 , 1.67185 , 1.68233 ,
     & 1.69128 , 1.70061 , 1.71066 , 1.7186 ,
     & 1.7273  , 1.73694 , 1.74532 , 1.75474 ,
     & 1.76237 , 1.77202 , 1.78041 , 1.78902 ,
     & 1.79638 , 1.80355 , 1.81277 , 1.82023 ,
     & 1.83005 , 1.83731 , 1.84499 , 1.85338 ,
     & 1.86032 , 1.86706 , 1.87315 , 1.88191/ 
C      
C     100 - 1000GeV
C
       DATA S6CCM/ 1.88882 , 1.96099 , 2.02619 , 2.08773 ,
     & 2.14314 , 2.19801 , 2.24749 , 2.29239 ,
     & 2.34113 , 2.37854 , 2.42414 , 2.46048 ,
     & 2.49839 , 2.53365 , 2.56948 , 2.60251 ,
     & 2.63102 , 2.66228 , 2.69322 , 2.72143 ,
     & 2.74987 , 2.7778  , 2.80294 , 2.82466 ,
     & 2.85209 , 2.8756  , 2.90127 , 2.92477 ,
     & 2.94328 , 2.9682  , 2.98951 , 3.01125 ,
     & 3.02967 , 3.0447  , 3.06721 , 3.08477 ,
     & 3.10212 , 3.1241  , 3.13722 , 3.15295 ,
     & 3.17428 , 3.192   , 3.20951 , 3.22272 ,
     & 3.23612 , 3.25264 , 3.2677  , 3.28799 ,
     & 3.29768 , 3.31589/

       DATA S7CCM/ 3.33167 , 3.34292 , 3.35198 , 3.36523 ,
     & 3.38182 , 3.39493 , 3.4085  , 3.42015 ,
     & 3.43453 , 3.44655 , 3.462   , 3.47517 ,
     & 3.48389 , 3.49626 , 3.51253 , 3.51772 ,
     & 3.52908 , 3.53698 , 3.54887 , 3.56053 ,
     & 3.57329 , 3.58138 , 3.58961 , 3.60328 ,
     & 3.61517 , 3.62143 , 3.64171 , 3.64393 ,
     & 3.65848 , 3.66477 , 3.67452 , 3.68372 ,
     & 3.68797 , 3.69928 , 3.71513 , 3.71869 ,
     & 3.73217 , 3.73861 , 3.74524 , 3.75874/ 
C
C     Tau neitrino
C
C     0 - 1GeV -DONE
C
       DATA S0CCT/0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA S1CCT/0.0000000,0.0000000,0.0000000,0.0000000,0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
C
C     1 - 10GeV - DONE 
C      
       DATA S2CCT/0 , 0 , 0 , 0 , 
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 , 
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 , 0 , 0 , 0 ,
     & 0 ,           1.53254e-08 , 2.09568e-07 , 2.53576e-06 ,
     & 3.17187e-06 , 3.84169e-06 , 5.50324e-06 , 5.5703e-06 ,
     & 8.56871e-06 , 9.72727e-06/

       DATA S3CCT/ 1.2581e-05 , 1.32519e-05 , 1.80309e-05 , 2.4458e-05 ,
     & 3.12132e-05 , 4.03086e-05 , 4.99932e-05 , 5.77657e-05 ,
     & 7.01351e-05 , 8.82333e-05 , 9.2187e-05  , 0.000110504 ,
     & 0.000136369 , 0.000155739 , 0.000186527 , 0.000213431 ,
     & 0.000246023 , 0.000278959 , 0.000308508 , 0.000335249 ,
     & 0.000380433 , 0.000423117 , 0.00047323  , 0.000532353 ,
     & 0.000587692 , 0.000662306 , 0.000748284 , 0.000829959 ,
     & 0.000890342 , 0.000967234 , 0.00107855  , 0.00119049 ,
     & 0.00127153  , 0.00137009  , 0.00149615  , 0.00163106 ,
     & 0.0017688   , 0.00188109  , 0.00202939  , 0.0021827/ 
C
C     10 - 100GeV
C
       DATA S4CCT/0.00228002 , 0.00415195 , 0.00672161 , 0.00994755 ,
     & 0.0138485 , 0.01832   , 0.02346   , 0.0289588 ,
     & 0.0350142 , 0.0413592 , 0.0480617 , 0.0551611 ,
     & 0.062288  , 0.0697755 , 0.0773915 , 0.085158 ,
     & 0.0931985 , 0.101183  , 0.109179  , 0.117491 ,
     & 0.125736  , 0.133851  , 0.142032  , 0.150219 ,
     & 0.15868   , 0.166916  , 0.17553   , 0.183772 ,
     & 0.192018  , 0.200191  , 0.208273  , 0.21651 ,
     & 0.224694  , 0.232788  , 0.240997  , 0.249018 ,
     & 0.257103  , 0.264782  , 0.272892  , 0.280766 ,
     & 0.288738  , 0.296481  , 0.304307  , 0.311843 , 
     & 0.319802  , 0.327323  , 0.334851  , 0.342145 , 
     & 0.350025  , 0.357267/

       DATA S5CCT/ 0.364534 , 0.37197 , 0.379232 , 0.386632 ,
     & 0.39336  , 0.400709 , 0.407767 , 0.414707 ,
     & 0.421559 , 0.428591 , 0.435815 , 0.442459 , 
     & 0.449293 , 0.4558   , 0.462589 , 0.468973 , 
     & 0.476019 , 0.482018 , 0.489031 , 0.495889 ,
     & 0.502143 , 0.508212 , 0.514322 , 0.520951 ,
     & 0.526758 , 0.533018 , 0.539273 , 0.544935 , 
     & 0.551769 , 0.557509 , 0.563743 , 0.569656 , 
     & 0.575844 , 0.581565 , 0.587598 , 0.593039 ,
     & 0.599177 , 0.604841 , 0.610598 , 0.616205/ 
C
C     100 - 1000GeV
C
       DATA S6CCT/ 0.621067 , 0.675823 , 0.726964 , 0.77514 ,
     & 0.821979 , 0.865537 , 0.906689 , 0.945212 , 
     & 0.984435 , 1.01995  , 1.05528  , 1.08831 ,
     & 1.12245  , 1.15336  , 1.18347  , 1.21246 ,
     & 1.24039  , 1.26787  , 1.29507  , 1.32152 , 
     & 1.34832  , 1.36996  , 1.39482  , 1.41758 ,
     & 1.44064  , 1.46365  , 1.48412  , 1.5075 ,
     & 1.52619  , 1.54712  , 1.5659   , 1.58672 ,
     & 1.60262  , 1.62203  , 1.64084  , 1.66028 ,
     & 1.67747  , 1.69459  , 1.71046  , 1.7275 ,
     & 1.74541  , 1.76227  , 1.77406  , 1.79038 ,
     & 1.80618  , 1.82195  , 1.83677  , 1.85021 ,
     & 1.86423  , 1.87973/

       DATA S7CCT/ 1.89076 , 1.90417 , 1.92066 , 1.93096 ,
     & 1.94388 , 1.95768 , 1.97026 , 1.98393 ,
     & 1.99661 , 2.00811 , 2.02194 , 2.03199 ,
     & 2.04529 , 2.05619 , 2.0664  , 2.07594 ,
     & 2.08914 , 2.10054 , 2.10828 , 2.12252 ,
     & 2.1358  , 2.14391 , 2.15439 , 2.16381 ,
     & 2.17357 , 2.18663 , 2.19292 , 2.2028 , 
     & 2.21351 , 2.22409 , 2.23458 , 2.24024 ,
     & 2.25109 , 2.26017 , 2.26809 , 2.27938 ,
     & 2.28676 , 2.29739 , 2.3074  , 2.31387/ 
C
C     Neutral current
C     ===============
C     0 - 1GeV - DONE
C      
       DATA S0NC/0 , -0.00484458 , -0.00347004 , -0.00239414 ,
     & -0.00157822  , -0.000984059 , -0.000573551 , -0.000309116 ,
     & -0.000151861 , -6.58294e-05 , -2.40507e-05 , -6.61236e-06 ,
     & -1.05643e-06 , -3.86747e-08 , 1.38079e-11  , 3.74455e-08 ,
     & 4.87345e-07  , 2.28396e-06  , 6.71798e-06  , 1.54802e-05 ,
     & 3.03617e-05  , 5.30807e-05  , 8.535e-05    , 0.000130177 ,
     & 0.000190175  , 0.000271968  , 0.000380883  , 0.000518807 ,
     & 0.000687409  , 0.000889836  , 0.0011209    , 0.00138489 ,
     & 0.00167842   , 0.00200374   , 0.00235952   , 0.00274042 , 
     & 0.00314904   , 0.00357073   , 0.00404088   , 0.00450574 , 
     & 0.00498561   , 0.00548621   , 0.00599684   , 0.00650747 ,
     & 0.0070382    , 0.00756873   , 0.00812261   , 0.00868395 ,
     & 0.00925837   , 0.00983859/

       DATA S1NC/ 0.0104287 , 0.0110292 , 0.0116318 , 0.0122307 , 
     & 0.0128259 , 0.0134336 , 0.0140434 , 0.0146576 , 
     & 0.0152506 , 0.015874  , 0.016504  , 0.0171146 ,
     & 0.01775   , 0.0183272 , 0.0188537 , 0.0194669 ,
     & 0.020067  , 0.0206931 , 0.0213141 , 0.021939 , 
     & 0.0225278 , 0.0231124 , 0.0237072 , 0.0242783 ,
     & 0.0248742 , 0.0254486 , 0.0260823 , 0.0266602 ,
     & 0.0272347 , 0.0278392 , 0.0284182 , 0.0289758 ,
     & 0.0295146 , 0.0300239 , 0.0306    , 0.0311092 , 
     & 0.0316285 , 0.0321341 , 0.032439  , 0.0329822 ,
     & 0.0334931 , 0.0339953 , 0.0345177 , 0.0350276 , 
     & 0.0355058 , 0.0360105 , 0.0364891 , 0.036964 ,
     & 0.0374473 , 0.0379444/ 
C            
C     1 - 10GeV - DONE
C      
       DATA S2NC/ 0.038639 , 0.0434933 , 0.048138 , 0.0526534 ,
     & 0.0569998 , 0.0612101 , 0.0653092 , 0.069465 ,
     & 0.0736318 , 0.077484  , 0.0812676 , 0.0850889 ,
     & 0.0888273 , 0.0925542 , 0.0962851 , 0.0999581 ,
     & 0.103832  , 0.107372  , 0.110855  , 0.11439 ,
     & 0.117818  , 0.121161  , 0.124432  , 0.127748 ,
     & 0.131068  , 0.133704  , 0.136893  , 0.140169 , 
     & 0.143332  , 0.146397  , 0.149487  , 0.152542 ,
     & 0.155559  , 0.158753  , 0.161979  , 0.165009 ,
     & 0.167785  , 0.170874  , 0.173703  , 0.176542 ,
     & 0.179417  , 0.182234  , 0.185205  , 0.188959 ,
     & 0.191726  , 0.194449  , 0.197178  , 0.199809 ,
     & 0.202577  , 0.205344/

       DATA S3NC/ 0.20795 , 0.209573 , 0.212117 , 0.214633 ,
     & 0.217153 , 0.219768 , 0.222311 , 0.224828 , 
     & 0.227314 , 0.22981  , 0.232806 , 0.235211 ,
     & 0.237688 , 0.24019  , 0.242598 , 0.245049 ,
     & 0.247413 , 0.249897 , 0.252278 , 0.252878 ,
     & 0.255233 , 0.257587 , 0.259929 , 0.262079 ,
     & 0.264303 , 0.266567 , 0.268834 , 0.271058 ,
     & 0.274221 , 0.276454 , 0.278586 , 0.280787 , 
     & 0.282896 , 0.28488  , 0.287028 , 0.289077 ,
     & 0.291097 , 0.292847 , 0.294886 , 0.296959/ 
C
C     10 - 100GeV
C
       DATA S4NC/ 0.300022 , 0.319974 , 0.339254 , 0.35732 ,
     & 0.37485  , 0.391569 , 0.407395 , 0.422919 ,
     & 0.437543 , 0.452158 , 0.465705 , 0.478851 ,
     & 0.491886 , 0.50487  , 0.516737 , 0.528422 , 
     & 0.540112 , 0.551329 , 0.562267 , 0.573028 ,
     & 0.58325  , 0.59346  , 0.603109 , 0.6127 ,
     & 0.622159 , 0.631509 , 0.640251 , 0.64921 ,
     & 0.657833 , 0.66662  , 0.674422 , 0.683076 ,
     & 0.690844 , 0.698728 , 0.706997 , 0.714262 , 
     & 0.721997 , 0.729214 , 0.736031 , 0.743237 , 
     & 0.750319 , 0.757263 , 0.763999 , 0.770801 , 
     & 0.7768   , 0.783746 , 0.789936 , 0.796716 ,
     & 0.80278  , 0.808544/

       DATA S5NC/ 0.815046 , 0.820762 , 0.826205 , 0.832315 ,
     & 0.837853 , 0.843189 , 0.8493 , 0.854853 ,
     & 0.860201 , 0.86544 , 0.871008 , 0.875854 ,
     & 0.881366 , 0.88616 , 0.891922 , 0.895744 ,
     & 0.901224 , 0.906159 , 0.910543 , 0.915959 ,
     & 0.920448 , 0.925042 , 0.930022 , 0.934498 ,
     & 0.939342 , 0.943389 , 0.948385 , 0.952268 ,
     & 0.957254 , 0.961015 , 0.965333 , 0.96925 ,
     & 0.973151 , 0.977939 , 0.981463 , 0.986227 ,
     & 0.989967 , 0.99379 , 0.998071 , 1.00219/ 
C            
C     100 - 1000GeV
C      
       DATA S6NC/ 1.00509 , 1.04272 , 1.07783 , 1.10888 ,
     & 1.13948 , 1.1681  , 1.1948  , 1.21992 ,
     & 1.24284 , 1.26615 , 1.28629 , 1.30726 ,
     & 1.32703 , 1.34512 , 1.36454 , 1.38173 , 
     & 1.39792 , 1.41357 , 1.42902 , 1.44551 , 
     & 1.45911 , 1.4738  , 1.48738 , 1.50007 , 
     & 1.51345 , 1.52717 , 1.53864 , 1.55016 ,
     & 1.56139 , 1.57316 , 1.58467 , 1.59655 ,
     & 1.60586 , 1.6175  , 1.62679 , 1.63531 ,
     & 1.64678 , 1.65618 , 1.66183 , 1.67435 ,
     & 1.68204 , 1.69152 , 1.69934 , 1.70787 , 
     & 1.7181  , 1.72487 , 1.73261 , 1.74156 ,
     & 1.74628 , 1.75554/

       DATA S7NC/ 1.76363 , 1.76887 , 1.77958 , 1.7862 ,
     & 1.79291 , 1.80209 , 1.80656 , 1.81534 , 
     & 1.81992 , 1.82518 , 1.83392 , 1.83992 , 
     & 1.84424 , 1.85065 , 1.85857 , 1.86532 ,
     & 1.86994 , 1.87307 , 1.8841  , 1.88924 ,
     & 1.89474 , 1.89794 , 1.90273 , 1.9094 ,
     & 1.91549 , 1.92249 , 1.92552 , 1.93207 ,
     & 1.93264 , 1.94156 , 1.94444 , 1.95471 ,
     & 1.95796 , 1.95907 , 1.96661 , 1.96975 ,
     & 1.97477 , 1.98193 , 1.98539 , 1.98806/ 

C      real*4 ecvtbl(51),crsmart(51),crsrs(51)
C      
C      data ecvtbl/0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,
C     $            1.0,1.1,1.2,1.3,1.4,1.5,1.6,1.7,1.8,1.9,
C     $            2.0,2.1,2.2,2.3,2.4,2.5,2.6,2.7,2.8,2.9,
C     $            3.0,3.1,3.2,3.3,3.4,3.5,3.6,3.7,3.8,3.9,
C     $            4.0,4.1,4.2,4.3,4.4,4.5,4.6,4.7,4.8,4.9,5.0/
C
C      data crsmart/0.0,0.0,0.0,0.0,0.0167,0.0489,0.0815,0.1127,0.1506,
C     $ 0.1880,0.2229,0.2512,0.2830,0.3135,0.3334,0.3782,0.4050,0.4335,
C     $ 0.4486,0.4721,0.4885,0.5142,0.5230,0.5467,0.5542,0.5696,0.5853,
C     $ 0.5954,0.6015,0.6119,0.6224,0.6374,0.6418,0.6462,0.6506,0.6550,
C     $ 0.6615,0.6679,0.6743,0.6807,0.6871,0.6919,0.6967,0.7014,0.7062,
C     $ 0.7110,0.7159,0.7208,0.7258,0.7307,0.7356/
C      
C      data crsrs/0.0,0.00000,0.00000,0.03435,0.09210,0.14570,0.19446,
C     $0.24492,0.28821,0.32446,0.35560,0.37092,0.39648,0.41947,0.44043,
C     $0.45992,0.47812,0.49501,0.51175,0.52668,0.54196,0.55593,0.56916,
C     $0.58207,0.59565,0.60761,0.61888,0.62966,0.64189,0.65381,0.66427,
C     $0.67351,0.68398,0.69312,0.70294,0.71363,0.72541,0.73382,0.73998,
C     $0.74900,0.75598,0.76388,0.77170,0.78064,0.79077,0.80010,0.80881,
C     $0.81511,0.81917,0.82684,0.83405/

      SIG1CCE(0) = 0.
      SIG1CCM(0) = 0.
      SIG1CCT(0) = 0.
      SIG1NC(0) = 0.

C     
C       DETERMINE COHCRS VALUE
C

C       ENERGY FROM 0GEV 1GEV  ( 0.01GEV/BIN )

      IF (E.LT.1.0) THEN
         WIDTH = 0.01
         I = INT(E/WIDTH)
         IF (I.GE.100) I=99
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG1CCE(I)
               SIGEND(2) = SIG1CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG1CCM(I)
               SIGEND(2) = SIG1CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG1CCT(I)
               SIGEND(2) = SIG1CCT(I+1)
            endif
         else
            SIGEND(1) = SIG1NC(I)
            SIGEND(2) = SIG1NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH
         EEND(2) = FLOAT(I+1)*WIDTH

C       ENERGY FROM 1GEV TO 10GEV  ( 0.1GEV/BIN )

      ELSE IF (E.LT.10.0E0) THEN
         ESTART = 1.
         WIDTH = 0.1
         I = INT((E-ESTART)/WIDTH)
         IF (I.GE.90) I=89
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG2CCE(I)
               SIGEND(2) = SIG2CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG2CCM(I)
               SIGEND(2) = SIG2CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG2CCT(I)
               SIGEND(2) = SIG2CCT(I+1)
            endif
         else
            SIGEND(1) = SIG2NC(I)
            SIGEND(2) = SIG2NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C       ENERGY FROM 10GEV TO 100GEV  ( 1GEV/BIN )

      ELSE IF (E.LT.100.0E0) THEN
         ESTART = 10.
         WIDTH = 1.
         I = INT((E-ESTART)/WIDTH)
         IF (I.GE.90) I=89
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG3CCE(I)
               SIGEND(2) = SIG3CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG3CCM(I)
               SIGEND(2) = SIG3CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG3CCT(I)
               SIGEND(2) = SIG3CCT(I+1)
            endif
         else
            SIGEND(1) = SIG3NC(I)
            SIGEND(2) = SIG3NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C       ENERGY FROM 100GEV TO 1000GEV  ( 10GEV/BIN )

      ELSE IF (E.LT.1000.0E0) THEN  
         ESTART = 100.
         WIDTH = 10.
         I = INT((E-ESTART)/WIDTH)
         IF (I.GE.90) I=89
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG4CCE(I)
               SIGEND(2) = SIG4CCE(I+1)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG4CCM(I)
               SIGEND(2) = SIG4CCM(I+1)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG4CCT(I)
               SIGEND(2) = SIG4CCT(I+1)
            endif
         else
            SIGEND(1) = SIG4NC(I)
            SIGEND(2) = SIG4NC(I+1)
         endif
         EEND(1) = FLOAT(I)*WIDTH+ESTART
         EEND(2) = FLOAT(I+1)*WIDTH+ESTART

C       ENERGY OVER 1000GEV
      ELSE
         if(IC.eq.1) then
            if(abs(IPAR).eq.12) then
               SIGEND(1) = SIG4CCE(90)
               SIGEND(2) = SIG4CCE(90)
            elseif(abs(IPAR).eq.14) then
               SIGEND(1) = SIG4CCM(90)
               SIGEND(2) = SIG4CCM(90)
            elseif(abs(IPAR).eq.16) then
               SIGEND(1) = SIG4CCT(90)
               SIGEND(2) = SIG4CCT(90)
            endif
         else
            SIGEND(1) = SIG4NC(90)
            SIGEND(2) = SIG4NC(90)
         endif
         EEND(1) = 1000.
         EEND(2) = 1001.

      END IF

      SLOPE = (SIGEND(2)-SIGEND(1))/(EEND(2)-EEND(1))
      ScalingFactor = (NUMATOM/12.) ! Scaling by A rule
      COHCRSNEBERGER = (SIGEND(1)+SLOPE*(E-EEND(1)))*ScalingFactor
      if (cohcrsneberger.le.0) then
         cohcrsneberger = 0
      endif
      RETURN
      END
