************************************************************************
*     ---------------------------------
      REAL FUNCTION DIFCRSNEREIN(E,IPAR,IC)
*     ---------------------------------
*
*     (Purpose)
*       CROSS SECTION /( 10**(-38) CM**2 )
*
*     (Input)
*       E     : INCIDENT NEUTRINO ENERGY ( GEV )
*       IPAR  : INCIDENT NEUTRINO ID
*       IC    : INTERACTION MODE
*                1 : C.C.
*                0 : N.C.
*     (Output)
*       DIFCRSNE :  CROSS SECTION in units of 10^-38 cm^2
*      
*     (Creation Date and Author)
*       1994.08.30 ; K.Okumura
*       1996.02.05 ; K.Kaneyuki   DIFCRS -> DIFCRSNE
*       1996.12.20 ; K.Okumura    add cross section from 100GeV to 1000GeV
*       2007.01.08 ; G.Mitsuka    Taking into account lepton mass in CC,
*                                 and rename into difcrsnerein.F
*       2007.11.05 ; G.Mitsuka    support target nucleus besides 16O
*
*       2009.11.18 ; Y.Hayato     Recalculated with lepton mass corr.
*       2016.02.09 ; R. Terri    Copied for diffractive pions
*     (Comment)
*
************************************************************************

      IMPLICIT NONE
#include "necard.h"
      INTEGER IPAR,IC,I
      REAL E,WIDTH,EEND(2),SIGEND(2),SLOPE,ESTART,SCALE
C     Charged Current
      REAL SIG1CCE(0:100),SIG2CCE(0:90),SIG3CCE(0:90),SIG4CCE(0:90)
      REAL SIG1CCM(0:100),SIG2CCM(0:90),SIG3CCM(0:90),SIG4CCM(0:90)
      REAL SIG1CCT(0:100),SIG2CCT(0:90),SIG3CCT(0:90),SIG4CCT(0:90)
      REAL S0CCE(50),S1CCE(50),S2CCE(50),S3CCE(40)
      REAL S4CCE(50),S5CCE(40),S6CCE(50),S7CCE(40)
      REAL S0CCM(50),S1CCM(50),S2CCM(50),S3CCM(40)
      REAL S4CCM(50),S5CCM(40),S6CCM(50),S7CCM(40)
      REAL S0CCT(50),S1CCT(50),S2CCT(50),S3CCT(40)
      REAL S4CCT(50),S5CCT(40),S6CCT(50),S7CCT(40)
C     Electron neutrino
      EQUIVALENCE (SIG2CCE(0),SIG1CCE(100))
      EQUIVALENCE (SIG3CCE(0),SIG2CCE(90))
      EQUIVALENCE (SIG4CCE(0),SIG3CCE(90))
      EQUIVALENCE (SIG1CCE(1), S0CCE(1))
      EQUIVALENCE (SIG1CCE(51),S1CCE(1))
      EQUIVALENCE (SIG2CCE(1), S2CCE(1))
      EQUIVALENCE (SIG2CCE(51),S3CCE(1))
      EQUIVALENCE (SIG3CCE(1), S4CCE(1))
      EQUIVALENCE (SIG3CCE(51),S5CCE(1))
      EQUIVALENCE (SIG4CCE(1), S6CCE(1))
      EQUIVALENCE (SIG4CCE(51),S7CCE(1))
C     Muon neutrino
      EQUIVALENCE (SIG2CCM(0),SIG1CCM(100))
      EQUIVALENCE (SIG3CCM(0),SIG2CCM(90))
      EQUIVALENCE (SIG4CCM(0),SIG3CCM(90))
      EQUIVALENCE (SIG1CCM(1), S0CCM(1))
      EQUIVALENCE (SIG1CCM(51),S1CCM(1))
      EQUIVALENCE (SIG2CCM(1), S2CCM(1))
      EQUIVALENCE (SIG2CCM(51),S3CCM(1))
      EQUIVALENCE (SIG3CCM(1), S4CCM(1))
      EQUIVALENCE (SIG3CCM(51),S5CCM(1))
      EQUIVALENCE (SIG4CCM(1), S6CCM(1))
      EQUIVALENCE (SIG4CCM(51),S7CCM(1))
C     Tau neutrino
      EQUIVALENCE (SIG2CCT(0),SIG1CCT(100))
      EQUIVALENCE (SIG3CCT(0),SIG2CCT(90))
      EQUIVALENCE (SIG4CCT(0),SIG3CCT(90))
      EQUIVALENCE (SIG1CCT(1), S0CCT(1))
      EQUIVALENCE (SIG1CCT(51),S1CCT(1))
      EQUIVALENCE (SIG2CCT(1), S2CCT(1))
      EQUIVALENCE (SIG2CCT(51),S3CCT(1))
      EQUIVALENCE (SIG3CCT(1), S4CCT(1))
      EQUIVALENCE (SIG3CCT(51),S5CCT(1))
      EQUIVALENCE (SIG4CCT(1), S6CCT(1))
      EQUIVALENCE (SIG4CCT(51),S7CCT(1))

      ! Neutral Current
      REAL SIG1NC(0:100),SIG2NC(0:90),SIG3NC(0:90),SIG4NC(0:90)
      REAL S0NC(50),S1NC(50),S2NC(50),S3NC(40)
      REAL S4NC(50),S5NC(40),S6NC(50),S7NC(40)
      EQUIVALENCE (SIG2NC(0),SIG1NC(100))
      EQUIVALENCE (SIG3NC(0),SIG2NC(90))
      EQUIVALENCE (SIG4NC(0),SIG3NC(90))
      EQUIVALENCE (SIG1NC(1), S0NC(1))
      EQUIVALENCE (SIG1NC(51),S1NC(1))
      EQUIVALENCE (SIG2NC(1), S2NC(1))
      EQUIVALENCE (SIG2NC(51),S3NC(1))
      EQUIVALENCE (SIG3NC(1), S4NC(1))
      EQUIVALENCE (SIG3NC(51),S5NC(1))
      EQUIVALENCE (SIG4NC(1), S6NC(1))
      EQUIVALENCE (SIG4NC(51),S7NC(1))

C     Charged Current
      REAL WIG1CCE(0:100),WIG2CCE(0:90),WIG3CCE(0:90),WIG4CCE(0:90)
      REAL WIG1CCM(0:100),WIG2CCM(0:90),WIG3CCM(0:90),WIG4CCM(0:90)
      REAL WIG1CCT(0:100),WIG2CCT(0:90),WIG3CCT(0:90),WIG4CCT(0:90)
      REAL W0CCE(50),W1CCE(50),W2CCE(50),W3CCE(40)
      REAL W4CCE(50),W5CCE(40),W6CCE(50),W7CCE(40)
      REAL W0CCM(50),W1CCM(50),W2CCM(50),W3CCM(40)
      REAL W4CCM(50),W5CCM(40),W6CCM(50),W7CCM(40)
      REAL W0CCT(50),W1CCT(50),W2CCT(50),W3CCT(40)
      REAL W4CCT(50),W5CCT(40),W6CCT(50),W7CCT(40)
C     Electron neutrino
      EQUIVALENCE (WIG2CCE(0),WIG1CCE(100))
      EQUIVALENCE (WIG3CCE(0),WIG2CCE(90))
      EQUIVALENCE (WIG4CCE(0),WIG3CCE(90))
      EQUIVALENCE (WIG1CCE(1), W0CCE(1))
      EQUIVALENCE (WIG1CCE(51),W1CCE(1))
      EQUIVALENCE (WIG2CCE(1), W2CCE(1))
      EQUIVALENCE (WIG2CCE(51),W3CCE(1))
      EQUIVALENCE (WIG3CCE(1), W4CCE(1))
      EQUIVALENCE (WIG3CCE(51),W5CCE(1))
      EQUIVALENCE (WIG4CCE(1), W6CCE(1))
      EQUIVALENCE (WIG4CCE(51),W7CCE(1))
C     Muon neutrino
      EQUIVALENCE (WIG2CCM(0),WIG1CCM(100))
      EQUIVALENCE (WIG3CCM(0),WIG2CCM(90))
      EQUIVALENCE (WIG4CCM(0),WIG3CCM(90))
      EQUIVALENCE (WIG1CCM(1), W0CCM(1))
      EQUIVALENCE (WIG1CCM(51),W1CCM(1))
      EQUIVALENCE (WIG2CCM(1), W2CCM(1))
      EQUIVALENCE (WIG2CCM(51),W3CCM(1))
      EQUIVALENCE (WIG3CCM(1), W4CCM(1))
      EQUIVALENCE (WIG3CCM(51),W5CCM(1))
      EQUIVALENCE (WIG4CCM(1), W6CCM(1))
      EQUIVALENCE (WIG4CCM(51),W7CCM(1))
C     Tau neutrino
      EQUIVALENCE (WIG2CCT(0),WIG1CCT(100))
      EQUIVALENCE (WIG3CCT(0),WIG2CCT(90))
      EQUIVALENCE (WIG4CCT(0),WIG3CCT(90))
      EQUIVALENCE (WIG1CCT(1), W0CCT(1))
      EQUIVALENCE (WIG1CCT(51),W1CCT(1))
      EQUIVALENCE (WIG2CCT(1), W2CCT(1))
      EQUIVALENCE (WIG2CCT(51),W3CCT(1))
      EQUIVALENCE (WIG3CCT(1), W4CCT(1))
      EQUIVALENCE (WIG3CCT(51),W5CCT(1))
      EQUIVALENCE (WIG4CCT(1), W6CCT(1))
      EQUIVALENCE (WIG4CCT(51),W7CCT(1))

      ! Neutral Current
      REAL WIG1NC(0:100),WIG2NC(0:90),WIG3NC(0:90),WIG4NC(0:90)
      REAL W0NC(50),W1NC(50),W2NC(50),W3NC(40)
      REAL W4NC(50),W5NC(40),W6NC(50),W7NC(40)
      EQUIVALENCE (WIG2NC(0),WIG1NC(100))
      EQUIVALENCE (WIG3NC(0),WIG2NC(90))
      EQUIVALENCE (WIG4NC(0),WIG3NC(90))
      EQUIVALENCE (WIG1NC(1), W0NC(1))
      EQUIVALENCE (WIG1NC(51),W1NC(1))
      EQUIVALENCE (WIG2NC(1), W2NC(1))
      EQUIVALENCE (WIG2NC(51),W3NC(1))
      EQUIVALENCE (WIG3NC(1), W4NC(1))
      EQUIVALENCE (WIG3NC(51),W5NC(1))
      EQUIVALENCE (WIG4NC(1), W6NC(1))
      EQUIVALENCE (WIG4NC(51),W7NC(1))

       DATA S0CCE/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000037,
     &  0.0000196,  0.0000508,  0.0000978,  0.0001611,  0.0002410,
     &  0.0003349,  0.0004428,  0.0005635,  0.0006977,  0.0008429,
     &  0.0009956,  0.0011601,  0.0013319,  0.0015083,  0.0016935,
     &  0.0018858,  0.0020754,  0.0022740,  0.0024769,  0.0026820/
       DATA S1CCE/0.0028947,  0.0031046,  0.0033306,  0.0035451,  0.0037614,
     &  0.0039830,  0.0042036,  0.0044342,  0.0046611,  0.0048924,
     &  0.0051227,  0.0053585,  0.0056007,  0.0058347,  0.0060703,
     &  0.0063070,  0.0065446,  0.0067816,  0.0070346,  0.0072631,
     &  0.0075085,  0.0077542,  0.0080026,  0.0082495,  0.0084878,
     &  0.0087260,  0.0089754,  0.0092207,  0.0094560,  0.0097122,
     &  0.0099593,  0.0102047,  0.0104440,  0.0106858,  0.0109287,
     &  0.0111795,  0.0114118,  0.0116446,  0.0118910,  0.0121262,
     &  0.0123784,  0.0126061,  0.0128687,  0.0130934,  0.0133254,
     &  0.0135834,  0.0138348,  0.0140680,  0.0143022,  0.0145490/
       DATA S2CCE/0.0169015,  0.0191043,  0.0212453,  0.0233214,  0.0253252,
     &  0.0273317,  0.0292934,  0.0312304,  0.0330950,  0.0349371,
     &  0.0367905,  0.0385569,  0.0403067,  0.0420208,  0.0436353,
     &  0.0452386,  0.0468868,  0.0485428,  0.0500318,  0.0515846,
     &  0.0530314,  0.0544144,  0.0556797,  0.0570304,  0.0583772,
     &  0.0595992,  0.0608996,  0.0622073,  0.0633604,  0.0645520,
     &  0.0657586,  0.0668696,  0.0681411,  0.0693660,  0.0703825,
     &  0.0715304,  0.0726641,  0.0737197,  0.0748938,  0.0758941,
     &  0.0769031,  0.0777773,  0.0788259,  0.0799557,  0.0808373,
     &  0.0819296,  0.0828102,  0.0839270,  0.0847054,  0.0855995/
       DATA S3CCE/0.0866487,  0.0875089,  0.0882715,  0.0894286,  0.0901925,
     &  0.0910236,  0.0919072,  0.0925766,  0.0935289,  0.0939861,
     &  0.0949416,  0.0957057,  0.0965231,  0.0970018,  0.0978395,
     &  0.0985835,  0.0992504,  0.0999454,  0.1008410,  0.1014590,
     &  0.1021180,  0.1028300,  0.1032750,  0.1041590,  0.1047930,
     &  0.1051600,  0.1060760,  0.1069750,  0.1075750,  0.1081850,
     &  0.1087750,  0.1094630,  0.1098510,  0.1105970,  0.1114200,
     &  0.1120080,  0.1123880,  0.1131450,  0.1135800,  0.1144420/
       DATA S4CCE/0.1199920,  0.1254190,  0.1305220,  0.1351760,  0.1391320,
     &  0.1428380,  0.1466410,  0.1503800,  0.1538770,  0.1569060,
     &  0.1600730,  0.1630540,  0.1660030,  0.1685710,  0.1718100,
     &  0.1742550,  0.1761830,  0.1789710,  0.1814260,  0.1835250,
     &  0.1855310,  0.1873850,  0.1894600,  0.1910000,  0.1925150,
     &  0.1948840,  0.1963430,  0.1981070,  0.1995550,  0.2010110,
     &  0.2028770,  0.2051680,  0.2068330,  0.2068770,  0.2086160,
     &  0.2116780,  0.2123500,  0.2138750,  0.2160300,  0.2175850,
     &  0.2183450,  0.2185650,  0.2207950,  0.2215790,  0.2235490,
     &  0.2239830,  0.2257700,  0.2261710,  0.2271630,  0.2287790/
       DATA S5CCE/0.2296980,  0.2323580,  0.2317350,  0.2325270,  0.2335580,
     &  0.2346100,  0.2362580,  0.2363270,  0.2385310,  0.2379940,
     &  0.2381730,  0.2402160,  0.2405750,  0.2415670,  0.2433590,
     &  0.2432080,  0.2460690,  0.2447460,  0.2468490,  0.2469230,
     &  0.2477580,  0.2479110,  0.2507730,  0.2504430,  0.2516330,
     &  0.2529930,  0.2540140,  0.2529840,  0.2534210,  0.2553010,
     &  0.2548860,  0.2542420,  0.2560370,  0.2561320,  0.2559030,
     &  0.2586590,  0.2604950,  0.2609690,  0.2584080,  0.2609210/
       DATA S6CCE/0.2686270,  0.2736990,  0.2793950,  0.2838980,  0.2883560,
     &  0.2912600,  0.2964410,  0.3006780,  0.3020050,  0.3081560,
     &  0.3116190,  0.3123830,  0.3188660,  0.3179470,  0.3221270,
     &  0.3267190,  0.3302230,  0.3282540,  0.3302950,  0.3333530,
     &  0.3367500,  0.3374340,  0.3399540,  0.3419890,  0.3416100,
     &  0.3439640,  0.3490920,  0.3485750,  0.3498670,  0.3506770,
     &  0.3517710,  0.3558220,  0.3591730,  0.3564320,  0.3612050,
     &  0.3614380,  0.3630290,  0.3643990,  0.3659690,  0.3694390,
     &  0.3681780,  0.3683700,  0.3710590,  0.3752270,  0.3737710,
     &  0.3742810,  0.3735740,  0.3729830,  0.3761570,  0.3785570/
       DATA S7CCE/0.3793870,  0.3808440,  0.3828330,  0.3805070,  0.3811300,
     &  0.3860980,  0.3844350,  0.3834150,  0.3856300,  0.3891830,
     &  0.3907250,  0.3898940,  0.3932960,  0.3880220,  0.3943240,
     &  0.3901340,  0.3931980,  0.3930510,  0.3933470,  0.3955830,
     &  0.3985430,  0.3953640,  0.4000000,  0.3997860,  0.4006630,
     &  0.4013430,  0.4041290,  0.4048060,  0.4011870,  0.4028620,
     &  0.4091620,  0.4080320,  0.4051530,  0.4043070,  0.4093340,
     &  0.4087380,  0.4063340,  0.4134280,  0.4096810,  0.4148000/

        DATA S0CCM/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000058,  0.0000273,  0.0000681,  0.0001295,  0.0002097,
     &  0.0003089,  0.0004240,  0.0005539,  0.0007046,  0.0008621/
        DATA S1CCM/0.0010346,  0.0012182,  0.0014082,  0.0016081,  0.0018139,
     &  0.0020254,  0.0022437,  0.0024666,  0.0026919,  0.0029243,
     &  0.0031533,  0.0033923,  0.0036273,  0.0038676,  0.0041047,
     &  0.0043468,  0.0045901,  0.0048334,  0.0050719,  0.0053209,
     &  0.0055654,  0.0058209,  0.0060776,  0.0063211,  0.0065799,
     &  0.0068179,  0.0070670,  0.0073234,  0.0075799,  0.0078236,
     &  0.0080728,  0.0083211,  0.0085726,  0.0088278,  0.0090778,
     &  0.0093151,  0.0095744,  0.0098322,  0.0100743,  0.0103294,
     &  0.0105796,  0.0108168,  0.0110686,  0.0113214,  0.0115554,
     &  0.0118005,  0.0120352,  0.0122895,  0.0125456,  0.0127746/
        DATA S2CCM/0.0151909,  0.0175148,  0.0196614,  0.0217506,  0.0237651,
     &  0.0257745,  0.0277033,  0.0296133,  0.0315220,  0.0332888,
     &  0.0350932,  0.0369325,  0.0385989,  0.0402956,  0.0419265,
     &  0.0435709,  0.0451464,  0.0467787,  0.0483089,  0.0498670,
     &  0.0512881,  0.0526731,  0.0540182,  0.0553125,  0.0567335,
     &  0.0579469,  0.0592052,  0.0603632,  0.0616269,  0.0628893,
     &  0.0640870,  0.0652194,  0.0664542,  0.0675238,  0.0687274,
     &  0.0698491,  0.0709062,  0.0719915,  0.0730502,  0.0743208,
     &  0.0752133,  0.0761465,  0.0771469,  0.0781943,  0.0793317,
     &  0.0803282,  0.0813147,  0.0821695,  0.0831070,  0.0841141/
        DATA S3CCM/0.0851359,  0.0859621,  0.0868491,  0.0877333,  0.0885046,
     &  0.0893934,  0.0902646,  0.0910542,  0.0917760,  0.0926217,
     &  0.0931436,  0.0940896,  0.0949805,  0.0952926,  0.0963210,
     &  0.0969179,  0.0976692,  0.0983596,  0.0992777,  0.0999261,
     &  0.1004020,  0.1013910,  0.1019480,  0.1026930,  0.1032430,
     &  0.1036760,  0.1047410,  0.1053550,  0.1059680,  0.1067490,
     &  0.1074680,  0.1079610,  0.1086790,  0.1092950,  0.1099600,
     &  0.1105970,  0.1110040,  0.1116080,  0.1123550,  0.1129730/
        DATA S4CCM/0.1184930,  0.1238310,  0.1295850,  0.1337420,  0.1378610,
     &  0.1417700,  0.1454200,  0.1490330,  0.1524780,  0.1562640,
     &  0.1592610,  0.1614410,  0.1649890,  0.1677640,  0.1697840,
     &  0.1720150,  0.1752980,  0.1775240,  0.1806970,  0.1827100,
     &  0.1845620,  0.1862880,  0.1880080,  0.1900880,  0.1916620,
     &  0.1936170,  0.1964610,  0.1968000,  0.1993480,  0.2001700,
     &  0.2025310,  0.2043140,  0.2046970,  0.2073960,  0.2083330,
     &  0.2099290,  0.2114550,  0.2132960,  0.2140460,  0.2151710,
     &  0.2169200,  0.2186200,  0.2197060,  0.2201560,  0.2214460,
     &  0.2228790,  0.2241110,  0.2249600,  0.2277870,  0.2294210/
        DATA S5CCM/0.2289680,  0.2304960,  0.2309640,  0.2321980,  0.2327400,
     &  0.2323600,  0.2341780,  0.2357000,  0.2378260,  0.2360050,
     &  0.2372830,  0.2389780,  0.2403870,  0.2426980,  0.2422400,
     &  0.2417280,  0.2434800,  0.2451840,  0.2443290,  0.2471680,
     &  0.2451900,  0.2493180,  0.2501520,  0.2492820,  0.2491860,
     &  0.2515550,  0.2535470,  0.2533880,  0.2522370,  0.2523340,
     &  0.2546500,  0.2559270,  0.2566390,  0.2570730,  0.2575870,
     &  0.2567610,  0.2588870,  0.2572140,  0.2603470,  0.2612410/
        DATA S6CCM/0.2676870,  0.2739480,  0.2778550,  0.2820730,  0.2869790,
     &  0.2917790,  0.2964130,  0.2983770,  0.3054340,  0.3050760,
     &  0.3078430,  0.3113320,  0.3158180,  0.3165130,  0.3217070,
     &  0.3249270,  0.3263020,  0.3278400,  0.3294440,  0.3333800,
     &  0.3338330,  0.3372710,  0.3400340,  0.3390790,  0.3441530,
     &  0.3431040,  0.3476450,  0.3480180,  0.3462710,  0.3527010,
     &  0.3522460,  0.3547870,  0.3549090,  0.3548000,  0.3610670,
     &  0.3590860,  0.3632470,  0.3659570,  0.3679630,  0.3667220,
     &  0.3681070,  0.3691780,  0.3699020,  0.3710100,  0.3711310,
     &  0.3713160,  0.3756800,  0.3761920,  0.3776970,  0.3752070/
        DATA S7CCM/0.3771960,  0.3786800,  0.3799630,  0.3825610,  0.3808050,
     &  0.3859180,  0.3884610,  0.3840790,  0.3837660,  0.3842370,
     &  0.3843010,  0.3887400,  0.3917540,  0.3903030,  0.3903220,
     &  0.3930600,  0.3955860,  0.3965690,  0.3947450,  0.3984400,
     &  0.3922360,  0.3977100,  0.3976870,  0.3984010,  0.4023080,
     &  0.3994100,  0.4019620,  0.4052010,  0.4055320,  0.4020240,
     &  0.4024700,  0.4066120,  0.4040930,  0.4046500,  0.4099550,
     &  0.4072630,  0.4081190,  0.4064260,  0.4081640,  0.4100020/

       DATA S0CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA S1CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA S2CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000007,
     &  0.0000035,  0.0000102,  0.0000228,  0.0000437,  0.0000739,
     &  0.0001168,  0.0001716,  0.0002417,  0.0003263,  0.0004273,
     &  0.0005404,  0.0006735,  0.0008246,  0.0009940,  0.0011734,
     &  0.0013707,  0.0015830,  0.0018114,  0.0020599,  0.0023061/
       DATA S3CCT/0.0025876,  0.0028766,  0.0031748,  0.0034583,  0.0037989,
     &  0.0041174,  0.0044665,  0.0048066,  0.0051885,  0.0055225,
     &  0.0059185,  0.0062987,  0.0066991,  0.0070896,  0.0074939,
     &  0.0079087,  0.0083115,  0.0087498,  0.0091218,  0.0095994,
     &  0.0100212,  0.0104492,  0.0108839,  0.0112939,  0.0117379,
     &  0.0122010,  0.0126322,  0.0130630,  0.0135432,  0.0139896,
     &  0.0144607,  0.0148660,  0.0153565,  0.0158353,  0.0162571,
     &  0.0167482,  0.0172062,  0.0176649,  0.0181497,  0.0185674/
       DATA S4CCT/0.0231680,  0.0276096,  0.0319865,  0.0362433,  0.0404287,
     &  0.0445046,  0.0481804,  0.0519705,  0.0556763,  0.0588351,
     &  0.0623393,  0.0653119,  0.0682793,  0.0713215,  0.0741446,
     &  0.0772406,  0.0798287,  0.0824600,  0.0850341,  0.0873178,
     &  0.0900223,  0.0922320,  0.0944508,  0.0972641,  0.0991593,
     &  0.1010930,  0.1035460,  0.1048460,  0.1070090,  0.1095090,
     &  0.1110620,  0.1126770,  0.1137810,  0.1164980,  0.1177750,
     &  0.1197320,  0.1214990,  0.1225170,  0.1245430,  0.1261640,
     &  0.1279760,  0.1288610,  0.1305300,  0.1319470,  0.1329460,
     &  0.1354050,  0.1365560,  0.1375690,  0.1383420,  0.1404890/
       DATA S5CCT/0.1417330,  0.1431530,  0.1442400,  0.1455450,  0.1467650,
     &  0.1478830,  0.1488810,  0.1505480,  0.1507720,  0.1529390,
     &  0.1531540,  0.1540590,  0.1560940,  0.1564030,  0.1572200,
     &  0.1586430,  0.1600040,  0.1604420,  0.1617180,  0.1625970,
     &  0.1626820,  0.1638240,  0.1653040,  0.1658300,  0.1669310,
     &  0.1668170,  0.1697970,  0.1703760,  0.1709860,  0.1710910,
     &  0.1729220,  0.1730440,  0.1749330,  0.1744900,  0.1752730,
     &  0.1771480,  0.1767180,  0.1784820,  0.1796410,  0.1800930/
       DATA S6CCT/0.1870950,  0.1931920,  0.2000510,  0.2060740,  0.2107160,
     &  0.2153350,  0.2203650,  0.2248630,  0.2280020,  0.2326950,
     &  0.2363770,  0.2393920,  0.2409000,  0.2474260,  0.2496470,
     &  0.2528800,  0.2546170,  0.2572480,  0.2592230,  0.2627960,
     &  0.2646800,  0.2671650,  0.2704540,  0.2706910,  0.2731230,
     &  0.2746800,  0.2778490,  0.2811130,  0.2807490,  0.2839020,
     &  0.2856800,  0.2873820,  0.2889470,  0.2915330,  0.2930250,
     &  0.2919760,  0.2978510,  0.2960150,  0.2991590,  0.2998630,
     &  0.3016920,  0.3040700,  0.3033510,  0.3057220,  0.3071260,
     &  0.3070920,  0.3091990,  0.3082510,  0.3102820,  0.3121720/
       DATA S7CCT/0.3138840,  0.3146540,  0.3181850,  0.3158450,  0.3164620,
     &  0.3182440,  0.3210510,  0.3218370,  0.3216180,  0.3222700,
     &  0.3234740,  0.3238290,  0.3262090,  0.3282010,  0.3266470,
     &  0.3285270,  0.3292060,  0.3312190,  0.3291210,  0.3326450,
     &  0.3355970,  0.3350130,  0.3351940,  0.3360260,  0.3349930,
     &  0.3371220,  0.3363400,  0.3388380,  0.3383460,  0.3403470,
     &  0.3405610,  0.3412930,  0.3429750,  0.3433850,  0.3435150,
     &  0.3471280,  0.3502960,  0.3450740,  0.3489430,  0.3465050/

       DATA S0NC/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000020,  0.0000103,
     &  0.0000262,  0.0000506,  0.0000835,  0.0001237,  0.0001721,
     &  0.0002267,  0.0002880,  0.0003562,  0.0004302,  0.0005088,
     &  0.0005910,  0.0006779,  0.0007676,  0.0008601,  0.0009573,
     &  0.0010535,  0.0011553,  0.0012562,  0.0013615,  0.0014674/
       DATA S1NC/0.0015761,  0.0016863,  0.0017975,  0.0019081,  0.0020208,
     &  0.0021334,  0.0022481,  0.0023656,  0.0024830,  0.0026010,
     &  0.0027200,  0.0028385,  0.0029584,  0.0030800,  0.0032034,
     &  0.0033223,  0.0034418,  0.0035649,  0.0036880,  0.0038124,
     &  0.0039355,  0.0040641,  0.0041830,  0.0043086,  0.0044307,
     &  0.0045592,  0.0046794,  0.0048020,  0.0049272,  0.0050566,
     &  0.0051703,  0.0053006,  0.0054282,  0.0055462,  0.0056688,
     &  0.0057923,  0.0059109,  0.0060335,  0.0061573,  0.0062807,
     &  0.0063998,  0.0065250,  0.0066461,  0.0067706,  0.0068907,
     &  0.0070174,  0.0071359,  0.0072657,  0.0073844,  0.0075003/
       DATA S2NC/0.0086977,  0.0098127,  0.0108902,  0.0119434,  0.0129712,
     &  0.0139648,  0.0149655,  0.0159353,  0.0169046,  0.0178069,
     &  0.0187402,  0.0196322,  0.0205053,  0.0213417,  0.0222089,
     &  0.0230050,  0.0238270,  0.0246372,  0.0254205,  0.0261805,
     &  0.0269051,  0.0275992,  0.0282238,  0.0289161,  0.0295881,
     &  0.0302232,  0.0308441,  0.0315342,  0.0320840,  0.0327187,
     &  0.0333015,  0.0338768,  0.0344786,  0.0351199,  0.0356628,
     &  0.0362529,  0.0367692,  0.0373271,  0.0378350,  0.0384213,
     &  0.0389170,  0.0393658,  0.0398763,  0.0403884,  0.0408851,
     &  0.0413854,  0.0419058,  0.0424072,  0.0428087,  0.0432387/
       DATA S3NC/0.0437463,  0.0441787,  0.0446271,  0.0451469,  0.0455482,
     &  0.0458837,  0.0462662,  0.0467512,  0.0471194,  0.0475529,
     &  0.0478889,  0.0482713,  0.0486437,  0.0489968,  0.0494187,
     &  0.0496854,  0.0500744,  0.0504853,  0.0508678,  0.0512554,
     &  0.0514918,  0.0519223,  0.0522691,  0.0525220,  0.0527374,
     &  0.0532164,  0.0535562,  0.0538297,  0.0542123,  0.0545726,
     &  0.0548573,  0.0552974,  0.0555462,  0.0559971,  0.0560979,
     &  0.0563556,  0.0566585,  0.0569212,  0.0574304,  0.0577581/
       DATA S4NC/0.0605412,  0.0631585,  0.0656969,  0.0677994,  0.0699026,
     &  0.0720702,  0.0738286,  0.0755926,  0.0773389,  0.0789836,
     &  0.0803476,  0.0817463,  0.0833458,  0.0850047,  0.0858410,
     &  0.0873623,  0.0886931,  0.0898728,  0.0910454,  0.0922151,
     &  0.0931385,  0.0942478,  0.0950372,  0.0958862,  0.0968897,
     &  0.0980660,  0.0985855,  0.0993930,  0.1005450,  0.1008940,
     &  0.1019160,  0.1030310,  0.1036960,  0.1046870,  0.1048280,
     &  0.1056700,  0.1066890,  0.1074060,  0.1083610,  0.1086770,
     &  0.1086530,  0.1097690,  0.1105280,  0.1113780,  0.1115710,
     &  0.1127540,  0.1133400,  0.1145820,  0.1144160,  0.1150480/
       DATA S5NC/0.1151240,  0.1157710,  0.1160300,  0.1169570,  0.1171220,
     &  0.1184150,  0.1185600,  0.1196880,  0.1193560,  0.1195080,
     &  0.1198050,  0.1207090,  0.1214040,  0.1209070,  0.1212660,
     &  0.1223090,  0.1231110,  0.1231750,  0.1228420,  0.1246650,
     &  0.1249360,  0.1249630,  0.1252090,  0.1259620,  0.1260110,
     &  0.1270020,  0.1266390,  0.1280870,  0.1291690,  0.1280210,
     &  0.1269330,  0.1285830,  0.1286540,  0.1290380,  0.1299640,
     &  0.1297970,  0.1299070,  0.1305450,  0.1313430,  0.1311080/
       DATA S6NC/0.1344610,  0.1376250,  0.1401030,  0.1425520,  0.1446780,
     &  0.1466700,  0.1480970,  0.1510730,  0.1516340,  0.1539970,
     &  0.1561110,  0.1562160,  0.1586110,  0.1605970,  0.1610760,
     &  0.1626670,  0.1639140,  0.1645530,  0.1662320,  0.1670740,
     &  0.1685840,  0.1693200,  0.1694400,  0.1717400,  0.1716500,
     &  0.1729590,  0.1723880,  0.1750650,  0.1751100,  0.1752280,
     &  0.1775620,  0.1781200,  0.1792750,  0.1779700,  0.1814950,
     &  0.1817800,  0.1818620,  0.1829670,  0.1835200,  0.1836870,
     &  0.1848320,  0.1858420,  0.1848130,  0.1873070,  0.1851950,
     &  0.1882490,  0.1870360,  0.1887280,  0.1878620,  0.1894510/
       DATA S7NC/0.1884750,  0.1909950,  0.1887470,  0.1917400,  0.1919740,
     &  0.1925750,  0.1926040,  0.1935460,  0.1943790,  0.1949980,
     &  0.1950510,  0.1948840,  0.1941460,  0.1948780,  0.1965130,
     &  0.1965320,  0.1970230,  0.1971790,  0.1993910,  0.1999830,
     &  0.1987680,  0.1990450,  0.1977950,  0.1997710,  0.2011530,
     &  0.2004080,  0.2020730,  0.2008580,  0.2013530,  0.2029090,
     &  0.2013160,  0.2035990,  0.2026060,  0.2051840,  0.2056160,
     &  0.2048610,  0.2062630,  0.2067220,  0.2046070,  0.2063180/

       DATA W0CCE/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W1CCE/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W2CCE/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000277,  0.0001094,
     &  0.0002462,  0.0004382,  0.0006772,  0.0009510,  0.0012626,
     &  0.0016133,  0.0019757,  0.0023895,  0.0028015,  0.0032318,
     &  0.0036685,  0.0041455,  0.0045891,  0.0050540,  0.0055538,
     &  0.0060765,  0.0065657,  0.0070671,  0.0075187,  0.0080499,
     &  0.0086058,  0.0091020,  0.0095810,  0.0101913,  0.0106801,
     &  0.0112369,  0.0117611,  0.0122850,  0.0127966,  0.0132608,
     &  0.0138158,  0.0143478,  0.0148835,  0.0154214,  0.0159503,
     &  0.0164701,  0.0170200,  0.0175512,  0.0181331,  0.0186480/
       DATA W3CCE/0.0191537,  0.0195884,  0.0200569,  0.0206536,  0.0212304,
     &  0.0217041,  0.0222249,  0.0226688,  0.0231255,  0.0236679,
     &  0.0241017,  0.0246888,  0.0251385,  0.0256454,  0.0261226,
     &  0.0265564,  0.0272047,  0.0275026,  0.0280574,  0.0284186,
     &  0.0289961,  0.0294564,  0.0298774,  0.0303347,  0.0307164,
     &  0.0311920,  0.0316295,  0.0321811,  0.0325465,  0.0330497,
     &  0.0336663,  0.0340092,  0.0345294,  0.0348484,  0.0352748,
     &  0.0357624,  0.0362123,  0.0366768,  0.0370420,  0.0376371/
       DATA W4CCE/0.0418038,  0.0455983,  0.0494574,  0.0532545,  0.0565577,
     &  0.0598260,  0.0629425,  0.0661909,  0.0690484,  0.0718208,
     &  0.0747865,  0.0773000,  0.0802743,  0.0826923,  0.0847495,
     &  0.0866816,  0.0893420,  0.0917357,  0.0937780,  0.0959124,
     &  0.0981064,  0.0998677,  0.1018800,  0.1042260,  0.1045150,
     &  0.1065760,  0.1090330,  0.1106850,  0.1114800,  0.1132360,
     &  0.1157760,  0.1167760,  0.1173660,  0.1196350,  0.1208950,
     &  0.1228440,  0.1243890,  0.1240600,  0.1263170,  0.1278030,
     &  0.1286430,  0.1306620,  0.1317700,  0.1315930,  0.1347040,
     &  0.1350820,  0.1369350,  0.1398880,  0.1390560,  0.1403540/
       DATA W5CCE/0.1419130,  0.1430610,  0.1438960,  0.1438320,  0.1448450,
     &  0.1479900,  0.1471470,  0.1473460,  0.1491920,  0.1499500,
     &  0.1501700,  0.1528320,  0.1533870,  0.1536390,  0.1544480,
     &  0.1556440,  0.1567240,  0.1573410,  0.1571130,  0.1597190,
     &  0.1622670,  0.1612310,  0.1620300,  0.1611960,  0.1635090,
     &  0.1646230,  0.1653920,  0.1649200,  0.1662300,  0.1655940,
     &  0.1654750,  0.1677880,  0.1693320,  0.1680680,  0.1679890,
     &  0.1726450,  0.1735510,  0.1719430,  0.1724570,  0.1744600/
       DATA W6CCE/0.1811020,  0.1877400,  0.1907750,  0.1965240,  0.2023670,
     &  0.2050750,  0.2098500,  0.2124790,  0.2172640,  0.2195640,
     &  0.2250690,  0.2272040,  0.2307790,  0.2350140,  0.2360600,
     &  0.2386590,  0.2416370,  0.2435580,  0.2456570,  0.2466820,
     &  0.2495930,  0.2537860,  0.2544080,  0.2585460,  0.2612880,
     &  0.2609910,  0.2636040,  0.2644190,  0.2645210,  0.2667320,
     &  0.2697370,  0.2722920,  0.2733040,  0.2759040,  0.2752340,
     &  0.2791200,  0.2795460,  0.2811030,  0.2783890,  0.2836210,
     &  0.2826110,  0.2838450,  0.2849210,  0.2871380,  0.2908650,
     &  0.2887740,  0.2907970,  0.2936710,  0.2927430,  0.2968040/
       DATA W7CCE/0.2951050,  0.2992270,  0.2977910,  0.2971220,  0.3009200,
     &  0.3007560,  0.3026880,  0.3057090,  0.3027250,  0.3034650,
     &  0.3034870,  0.3073540,  0.3037030,  0.3101640,  0.3079140,
     &  0.3088140,  0.3132250,  0.3160740,  0.3119530,  0.3096830,
     &  0.3138440,  0.3146840,  0.3166480,  0.3165370,  0.3175710,
     &  0.3204820,  0.3190990,  0.3184990,  0.3210450,  0.3221510,
     &  0.3214480,  0.3271050,  0.3216940,  0.3246160,  0.3253710,
     &  0.3276940,  0.3242360,  0.3271830,  0.3251050,  0.3296470/

       DATA W0CCM/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W1CCM/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W2CCM/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000107,
     &  0.0000884,  0.0002396,  0.0004452,  0.0006933,  0.0009871,
     &  0.0013035,  0.0016508,  0.0020475,  0.0024526,  0.0028937,
     &  0.0033393,  0.0037661,  0.0042361,  0.0047078,  0.0051722,
     &  0.0056640,  0.0061484,  0.0066172,  0.0071562,  0.0076646,
     &  0.0081378,  0.0086357,  0.0091534,  0.0096726,  0.0101955,
     &  0.0107151,  0.0112771,  0.0117970,  0.0123354,  0.0128002,
     &  0.0133401,  0.0138407,  0.0143561,  0.0149700,  0.0154679,
     &  0.0159513,  0.0164518,  0.0170620,  0.0175534,  0.0179950/
       DATA W3CCM/0.0186075,  0.0191370,  0.0195876,  0.0200884,  0.0206267,
     &  0.0211170,  0.0217070,  0.0221485,  0.0226633,  0.0230511,
     &  0.0236439,  0.0240767,  0.0246568,  0.0249637,  0.0255669,
     &  0.0260760,  0.0264691,  0.0269075,  0.0273268,  0.0278042,
     &  0.0282973,  0.0288684,  0.0293338,  0.0296683,  0.0302253,
     &  0.0305496,  0.0310781,  0.0315802,  0.0319538,  0.0325077,
     &  0.0330611,  0.0334829,  0.0337740,  0.0344026,  0.0346534,
     &  0.0352288,  0.0356091,  0.0359003,  0.0363906,  0.0367920/
       DATA W4CCM/0.0409753,  0.0449787,  0.0488251,  0.0525496,  0.0556535,
     &  0.0592239,  0.0624789,  0.0655842,  0.0683748,  0.0710510,
     &  0.0736839,  0.0762718,  0.0788787,  0.0812252,  0.0841656,
     &  0.0863879,  0.0884659,  0.0903121,  0.0929234,  0.0950172,
     &  0.0968864,  0.0987773,  0.1009350,  0.1028610,  0.1039750,
     &  0.1055930,  0.1072390,  0.1101500,  0.1110160,  0.1123120,
     &  0.1139550,  0.1155670,  0.1169990,  0.1181460,  0.1211480,
     &  0.1213100,  0.1228890,  0.1234630,  0.1258000,  0.1278640,
     &  0.1292420,  0.1301290,  0.1319410,  0.1327590,  0.1327940,
     &  0.1351370,  0.1358520,  0.1366650,  0.1389550,  0.1391520/
       DATA W5CCM/0.1400850,  0.1417120,  0.1421920,  0.1429530,  0.1458920,
     &  0.1457900,  0.1473270,  0.1469710,  0.1480400,  0.1515040,
     &  0.1498470,  0.1522250,  0.1526410,  0.1526960,  0.1545040,
     &  0.1547040,  0.1555200,  0.1553540,  0.1566790,  0.1586470,
     &  0.1605790,  0.1599080,  0.1608940,  0.1630230,  0.1630560,
     &  0.1632280,  0.1632240,  0.1640160,  0.1636660,  0.1656440,
     &  0.1662670,  0.1686360,  0.1699400,  0.1709600,  0.1717260,
     &  0.1710870,  0.1720690,  0.1702900,  0.1730330,  0.1736660/
       DATA W6CCM/0.1793310,  0.1854850,  0.1904210,  0.1952540,  0.2022580,
     &  0.2045980,  0.2078310,  0.2141520,  0.2171340,  0.2198400,
     &  0.2232900,  0.2283190,  0.2314840,  0.2337290,  0.2359270,
     &  0.2377230,  0.2406070,  0.2436460,  0.2465030,  0.2469660,
     &  0.2504280,  0.2539910,  0.2539760,  0.2565290,  0.2574790,
     &  0.2607410,  0.2638880,  0.2639300,  0.2663080,  0.2680450,
     &  0.2711160,  0.2709550,  0.2737070,  0.2739290,  0.2731160,
     &  0.2777420,  0.2804470,  0.2810360,  0.2842300,  0.2836780,
     &  0.2835060,  0.2855360,  0.2891030,  0.2876480,  0.2880270,
     &  0.2880900,  0.2914450,  0.2931220,  0.2947850,  0.2956720/
       DATA W7CCM/0.2949840,  0.2980260,  0.2985910,  0.2980100,  0.2962420,
     &  0.3014040,  0.3022780,  0.2992510,  0.3038420,  0.3044560,
     &  0.3111230,  0.3063400,  0.3043690,  0.3094990,  0.3095010,
     &  0.3125780,  0.3118160,  0.3086570,  0.3100160,  0.3154350,
     &  0.3170940,  0.3186660,  0.3161910,  0.3141190,  0.3187020,
     &  0.3225670,  0.3200670,  0.3176620,  0.3216010,  0.3247110,
     &  0.3235330,  0.3219730,  0.3196270,  0.3259390,  0.3267890,
     &  0.3255220,  0.3290600,  0.3304260,  0.3285570,  0.3298280/

       DATA W0CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W1CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W2CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W3CCT/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000014,  0.0000064,  0.0000141,  0.0000257,
     &  0.0000394,  0.0000576,  0.0000790,  0.0001041,  0.0001323,
     &  0.0001613,  0.0001970,  0.0002376,  0.0002795,  0.0003229,
     &  0.0003767,  0.0004294,  0.0004864,  0.0005397,  0.0006126,
     &  0.0006719,  0.0007434,  0.0008190,  0.0008886,  0.0009715,
     &  0.0010509,  0.0011437,  0.0012167,  0.0013165,  0.0014102/
       DATA W4CCT/0.0024858,  0.0037472,  0.0051660,  0.0067250,  0.0084097,
     &  0.0101351,  0.0117394,  0.0135141,  0.0153796,  0.0171497,
     &  0.0187342,  0.0207673,  0.0223771,  0.0241294,  0.0259753,
     &  0.0274810,  0.0292450,  0.0309641,  0.0325700,  0.0338535,
     &  0.0355823,  0.0372931,  0.0387243,  0.0401788,  0.0419107,
     &  0.0431671,  0.0448232,  0.0461902,  0.0471683,  0.0490937,
     &  0.0500845,  0.0512723,  0.0526470,  0.0540818,  0.0561267,
     &  0.0564867,  0.0574260,  0.0593946,  0.0602857,  0.0619659,
     &  0.0627087,  0.0644025,  0.0647113,  0.0663295,  0.0672970,
     &  0.0682362,  0.0697539,  0.0711522,  0.0720703,  0.0725055/
       DATA W5CCT/0.0733913,  0.0741491,  0.0757363,  0.0764637,  0.0779839,
     &  0.0779923,  0.0800365,  0.0804451,  0.0813645,  0.0831691,
     &  0.0838767,  0.0839578,  0.0847550,  0.0857250,  0.0869488,
     &  0.0876344,  0.0888500,  0.0896603,  0.0901249,  0.0906773,
     &  0.0915984,  0.0934240,  0.0930452,  0.0950118,  0.0957120,
     &  0.0962725,  0.0968523,  0.0973958,  0.0986007,  0.0980738,
     &  0.0998640,  0.0998013,  0.1011260,  0.1013590,  0.1027210,
     &  0.1033640,  0.1037590,  0.1038490,  0.1052010,  0.1062630/
       DATA W6CCT/0.1125700,  0.1190850,  0.1242250,  0.1294460,  0.1347050,
     &  0.1384800,  0.1423400,  0.1467620,  0.1499950,  0.1537930,
     &  0.1559190,  0.1612300,  0.1640000,  0.1672370,  0.1712040,
     &  0.1726910,  0.1751860,  0.1779600,  0.1796290,  0.1840020,
     &  0.1858030,  0.1880110,  0.1888730,  0.1925040,  0.1932980,
     &  0.1968400,  0.1977820,  0.2009310,  0.2022690,  0.2024810,
     &  0.2054460,  0.2062930,  0.2081990,  0.2101780,  0.2118220,
     &  0.2120540,  0.2146100,  0.2158180,  0.2177560,  0.2209090,
     &  0.2208160,  0.2213380,  0.2246040,  0.2248710,  0.2248960,
     &  0.2257700,  0.2275950,  0.2306510,  0.2307140,  0.2311480/
       DATA W7CCT/0.2328220,  0.2335910,  0.2348190,  0.2379460,  0.2361370,
     &  0.2388740,  0.2390900,  0.2406470,  0.2410680,  0.2411940,
     &  0.2425270,  0.2436910,  0.2444140,  0.2466780,  0.2465820,
     &  0.2486840,  0.2489570,  0.2501840,  0.2508220,  0.2504020,
     &  0.2544640,  0.2533300,  0.2532910,  0.2566500,  0.2546000,
     &  0.2570840,  0.2580170,  0.2568390,  0.2589180,  0.2603070,
     &  0.2612790,  0.2607950,  0.2636120,  0.2637150,  0.2638980,
     &  0.2646430,  0.2658640,  0.2673460,  0.2661360,  0.2674870/

       DATA W0NC/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W1NC/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000/
       DATA W2NC/0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,
     &  0.0000000,  0.0000000,  0.0000000,  0.0000145,  0.0000565,
     &  0.0001264,  0.0002214,  0.0003393,  0.0004796,  0.0006359,
     &  0.0008060,  0.0009888,  0.0011891,  0.0014012,  0.0016171,
     &  0.0018455,  0.0020682,  0.0022993,  0.0025404,  0.0027920,
     &  0.0030310,  0.0032915,  0.0035277,  0.0037823,  0.0040463,
     &  0.0043106,  0.0045684,  0.0048199,  0.0050681,  0.0053529,
     &  0.0056068,  0.0058707,  0.0061033,  0.0063925,  0.0066723,
     &  0.0069269,  0.0071729,  0.0074445,  0.0077300,  0.0079725,
     &  0.0082495,  0.0085492,  0.0087603,  0.0090616,  0.0093190/
       DATA W3NC/0.0095137,  0.0097983,  0.0100488,  0.0103766,  0.0105694,
     &  0.0107938,  0.0110485,  0.0113874,  0.0116083,  0.0118787,
     &  0.0121429,  0.0122934,  0.0125898,  0.0128677,  0.0130665,
     &  0.0133268,  0.0135403,  0.0137718,  0.0140747,  0.0142195,
     &  0.0144986,  0.0147232,  0.0148627,  0.0151775,  0.0154036,
     &  0.0155883,  0.0158698,  0.0160818,  0.0163547,  0.0165602,
     &  0.0167750,  0.0169769,  0.0172905,  0.0173998,  0.0176835,
     &  0.0179349,  0.0181561,  0.0183467,  0.0185165,  0.0187661/
       DATA W4NC/0.0207695,  0.0228450,  0.0247474,  0.0267088,  0.0283492,
     &  0.0299019,  0.0316679,  0.0331149,  0.0344119,  0.0358465,
     &  0.0371981,  0.0389356,  0.0397456,  0.0412788,  0.0425280,
     &  0.0433638,  0.0446253,  0.0459267,  0.0469389,  0.0478310,
     &  0.0487514,  0.0497259,  0.0506010,  0.0518912,  0.0527796,
     &  0.0535879,  0.0541463,  0.0554535,  0.0556819,  0.0569443,
     &  0.0573450,  0.0583790,  0.0590620,  0.0597700,  0.0607544,
     &  0.0613804,  0.0617780,  0.0633780,  0.0630771,  0.0637516,
     &  0.0648567,  0.0650288,  0.0660977,  0.0664779,  0.0670841,
     &  0.0675714,  0.0689347,  0.0693956,  0.0694628,  0.0702362/
       DATA W5NC/0.0706385,  0.0714599,  0.0716113,  0.0732452,  0.0728584,
     &  0.0734719,  0.0740843,  0.0738781,  0.0747799,  0.0756887,
     &  0.0754100,  0.0763736,  0.0770394,  0.0767195,  0.0773219,
     &  0.0774235,  0.0787334,  0.0787819,  0.0782580,  0.0796973,
     &  0.0802525,  0.0807448,  0.0812930,  0.0811696,  0.0819412,
     &  0.0819633,  0.0821919,  0.0826473,  0.0827239,  0.0838374,
     &  0.0834456,  0.0839588,  0.0846531,  0.0847046,  0.0846049,
     &  0.0858027,  0.0857087,  0.0859515,  0.0881739,  0.0877076/
       DATA W6NC/0.0899928,  0.0938103,  0.0962676,  0.0983891,  0.1009380,
     &  0.1028900,  0.1049180,  0.1072710,  0.1087220,  0.1119170,
     &  0.1128890,  0.1140230,  0.1162010,  0.1180240,  0.1184250,
     &  0.1195740,  0.1205740,  0.1215320,  0.1229360,  0.1249000,
     &  0.1250230,  0.1242510,  0.1275220,  0.1293890,  0.1292580,
     &  0.1302290,  0.1311710,  0.1324570,  0.1338470,  0.1336070,
     &  0.1352790,  0.1353740,  0.1363870,  0.1379530,  0.1383640,
     &  0.1374150,  0.1396920,  0.1402600,  0.1415490,  0.1421570,
     &  0.1422040,  0.1434120,  0.1437950,  0.1445710,  0.1443670,
     &  0.1448060,  0.1467380,  0.1458690,  0.1472900,  0.1462200/
       DATA W7NC/0.1478230,  0.1479020,  0.1477480,  0.1477940,  0.1497310,
     &  0.1510660,  0.1509910,  0.1521620,  0.1519740,  0.1523370,
     &  0.1547630,  0.1521210,  0.1533510,  0.1542410,  0.1554680,
     &  0.1553750,  0.1561200,  0.1532250,  0.1552880,  0.1557580,
     &  0.1573310,  0.1570770,  0.1581840,  0.1591800,  0.1580160,
     &  0.1590880,  0.1595850,  0.1588660,  0.1610340,  0.1621630,
     &  0.1613050,  0.1625010,  0.1622700,  0.1638120,  0.1626340,
     &  0.1626280,  0.1641500,  0.1632730,  0.1644940,  0.1639130/

      SIG1CCE(0) = 0.
      SIG1CCM(0) = 0.
      SIG1CCT(0) = 0.
      SIG1NC(0) = 0.

      WIG1CCE(0) = 0.
      WIG1CCM(0) = 0.
      WIG1CCT(0) = 0.
      WIG1NC(0) = 0.



C     
C       DETERMINE DIFCRS VALUE
C

C       ENERGY FROM 0GEV 1GEV  ( 0.01GEV/BIN )

      if (NEDIFPI .eq.1) THEN
         IF (E.LT.1.0) THEN
            WIDTH = 0.01
            I = INT(E/WIDTH)
            IF (I.GE.100) I=99
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = SIG1CCE(I)
                  SIGEND(2) = SIG1CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = SIG1CCM(I)
                  SIGEND(2) = SIG1CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = SIG1CCT(I)
                  SIGEND(2) = SIG1CCT(I+1)
               endif
            else
               SIGEND(1) = SIG1NC(I)
               SIGEND(2) = SIG1NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH
            EEND(2) = FLOAT(I+1)*WIDTH
            
C     ENERGY FROM 1GEV TO 10GEV  ( 0.1GEV/BIN )
            
         ELSE IF (E.LT.10.0E0) THEN
            ESTART = 1.
            WIDTH = 0.1
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = SIG2CCE(I)
                  SIGEND(2) = SIG2CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = SIG2CCM(I)
                  SIGEND(2) = SIG2CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = SIG2CCT(I)
                  SIGEND(2) = SIG2CCT(I+1)
               endif
            else
               SIGEND(1) = SIG2NC(I)
               SIGEND(2) = SIG2NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY FROM 10GEV TO 100GEV  ( 1GEV/BIN )
            
         ELSE IF (E.LT.100.0E0) THEN
            ESTART = 10.
            WIDTH = 1.
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = SIG3CCE(I)
                  SIGEND(2) = SIG3CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = SIG3CCM(I)
                  SIGEND(2) = SIG3CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = SIG3CCT(I)
                  SIGEND(2) = SIG3CCT(I+1)
               endif
            else
               SIGEND(1) = SIG3NC(I)
               SIGEND(2) = SIG3NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY FROM 100GEV TO 1000GEV  ( 10GEV/BIN )
            
         ELSE IF (E.LT.1000.0E0) THEN  
            ESTART = 100.
            WIDTH = 10.
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = SIG4CCE(I)
                  SIGEND(2) = SIG4CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = SIG4CCM(I)
                  SIGEND(2) = SIG4CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = SIG4CCT(I)
                  SIGEND(2) = SIG4CCT(I+1)
               endif
            else
               SIGEND(1) = SIG4NC(I)
               SIGEND(2) = SIG4NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY OVER 1000GEV
         ELSE
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = SIG4CCE(90)
                  SIGEND(2) = SIG4CCE(90)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = SIG4CCM(90)
                  SIGEND(2) = SIG4CCM(90)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = SIG4CCT(90)
                  SIGEND(2) = SIG4CCT(90)
               endif
            else
               SIGEND(1) = SIG4NC(90)
               SIGEND(2) = SIG4NC(90)
            endif
            EEND(1) = 1000.
            EEND(2) = 1001.

         END IF
      else if (NEDIFPI .eq. 0) THEN
         IF (E.LT.1.0) THEN
            WIDTH = 0.01
            I = INT(E/WIDTH)
            IF (I.GE.100) I=99
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = WIG1CCE(I)
                  SIGEND(2) = WIG1CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = WIG1CCM(I)
                  SIGEND(2) = WIG1CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = WIG1CCT(I)
                  SIGEND(2) = WIG1CCT(I+1)
               endif
            else
               SIGEND(1) = WIG1NC(I)
               SIGEND(2) = WIG1NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH
            EEND(2) = FLOAT(I+1)*WIDTH
            
C     ENERGY FROM 1GEV TO 10GEV  ( 0.1GEV/BIN )
            
         ELSE IF (E.LT.10.0E0) THEN
            ESTART = 1.
            WIDTH = 0.1
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = WIG2CCE(I)
                  SIGEND(2) = WIG2CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = WIG2CCM(I)
                  SIGEND(2) = WIG2CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = WIG2CCT(I)
                  SIGEND(2) = WIG2CCT(I+1)
               endif
            else
               SIGEND(1) = WIG2NC(I)
               SIGEND(2) = WIG2NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY FROM 10GEV TO 100GEV  ( 1GEV/BIN )
            
         ELSE IF (E.LT.100.0E0) THEN
            ESTART = 10.
            WIDTH = 1.
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = WIG3CCE(I)
                  SIGEND(2) = WIG3CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = WIG3CCM(I)
                  SIGEND(2) = WIG3CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = WIG3CCT(I)
                  SIGEND(2) = WIG3CCT(I+1)
               endif
            else
               SIGEND(1) = WIG3NC(I)
               SIGEND(2) = WIG3NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY FROM 100GEV TO 1000GEV  ( 10GEV/BIN )
            
         ELSE IF (E.LT.1000.0E0) THEN  
            ESTART = 100.
            WIDTH = 10.
            I = INT((E-ESTART)/WIDTH)
            IF (I.GE.90) I=89
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = WIG4CCE(I)
                  SIGEND(2) = WIG4CCE(I+1)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = WIG4CCM(I)
                  SIGEND(2) = WIG4CCM(I+1)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = WIG4CCT(I)
                  SIGEND(2) = WIG4CCT(I+1)
               endif
            else
               SIGEND(1) = WIG4NC(I)
               SIGEND(2) = WIG4NC(I+1)
            endif
            EEND(1) = FLOAT(I)*WIDTH+ESTART
            EEND(2) = FLOAT(I+1)*WIDTH+ESTART
            
C     ENERGY OVER 1000GEV
         ELSE
            if(IC.eq.1) then
               if(abs(IPAR).eq.12) then
                  SIGEND(1) = WIG4CCE(90)
                  SIGEND(2) = WIG4CCE(90)
               elseif(abs(IPAR).eq.14) then
                  SIGEND(1) = WIG4CCM(90)
                  SIGEND(2) = WIG4CCM(90)
               elseif(abs(IPAR).eq.16) then
                  SIGEND(1) = WIG4CCT(90)
                  SIGEND(2) = WIG4CCT(90)
               endif
            else
               SIGEND(1) = WIG4NC(90)
               SIGEND(2) = WIG4NC(90)
            endif
            EEND(1) = 1000.
            EEND(2) = 1001.

         END IF
      end if

      SLOPE = (SIGEND(2)-SIGEND(1))/(EEND(2)-EEND(1))

      if (NEDIFPI.eq.0) then
         SCALE = 0.68
      else if (NEDIFPI.eq.1) then
         SCALE = 0.6
      else
         SCALE = 0
      endif
      
      DIFCRSNEREIN = SCALE*(SIGEND(1)+SLOPE*(E-EEND(1)))

      RETURN
      END
